/*
 * AppointEditor.java
 * 
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 */

package schedule;

import java.awt.*;
import java.awt.event.*;
import java.util.*;


/**
 * AppointEditor
 *     A|Cgg̐VKo^AύXA폜s
 * @version    1.1 98/05/07
 * @auther     EMOTO Masahiko, Nihon Sun Microsystems K.K.
 */
class AppointEditor extends Dialog {
    Vector appList;
    ScheduleEditor editor;
    GridBagLayout gbl;
    GridBagConstraints gbc;
    Appointment app;
    TextField fieldDescription;
    TextField fieldStart;
    TextField fieldEnd;
    TextArea  areaNote;
    Button buttonDone;
    Button buttonCancel;
    Button buttonDelete;
    Panel panel;
    int year;
    int month;
    int day;
    
    class DateFormatException extends Exception {
	public DateFormatException(){
	    super("");
	}
    }
    
    /**
     *  A|gg̏CE폜
     *  @param al  A|Cgg̃Xg
     *  @param ap  ҏWΏۃA|Cgg
     */
    void editAppointment(Vector al,Appointment ap){
	int i;
	appList = al;
	app = ap;
	fieldStart.setText(""+form(app.start/60)+":"+form(app.start%60));
	fieldEnd.setText(""+form(app.end/60)+":"+form(app.end%60));
	fieldDescription.setText(app.description);
	areaNote.setText(app.note);
	setVisible(true);
    }
    
    /**
     * printf("%02d",x)
     *  @param v  
     *
     */
    String form(int v){
	if ( v<10 )
	    return "0"+v;
	else
	    return ""+v;
    }
    
    /**
     *  A|gg̐VKo^
     *  @param al A|CggXg
     *  @param y  N
     *  @param m  
     *  @param d  
     *  @param sh Jn
     *  @param sm Jn
     *  @param eh I
     *  @param em I
     */
    void registerAppointment(Vector al,
			     int y,int m,int d,
			     int sh,int sm,
			     int eh,int em){
	int i;
	appList = al;
	
	app = null;
	
	fieldStart.setText(""+form(sh) + ":" + form(sm));
	fieldEnd.setText(""+ form(eh) +":"+form(em));
	fieldDescription.setText("");
	areaNote.setText("");
	year = y;
	month = m;
	day = d;
	setVisible(true);
	
    }
    
    void place (Component c,int x,int y,int w,int a,int f) {
	gbc.gridx = x;
	gbc.gridy = y;
	gbc.gridwidth = w;
	gbc.anchor = a;
	gbc.fill = f;
	gbl.setConstraints(c,gbc);
	panel.add(c,gbc);
    }
    
    /**
     * hh:mm `̕ǂ݁A00:00 ̎Ԃ𕪒Pʂ
     * ԂB
     */
    int parseTime(String s) throws DateFormatException{
	char c;
	int h;
	int m;
	s.trim();
	int len = s.length();
	int i = 0;
	StringBuffer hString = new StringBuffer();
	StringBuffer mString = new StringBuffer();
	try {
	    c = s.charAt(i);
	    while ( '0' <= c && c <= '9' ){
		hString.append(c);
		c = s.charAt(++i);
	    }
	    while ( ! ( '0' <= c && c <= '9' )){
		c = s.charAt(++i);
	    }
	    while ( '0' <= c && c <= '9' ){
		mString.append(c);
		c = s.charAt(++i);
	    }
	}catch (StringIndexOutOfBoundsException ex){
	}
	if ( hString.equals("") ){
	    throw (new DateFormatException());
	}else{
	    try {
		h = Integer.parseInt(hString.toString());
	    }catch (NumberFormatException ex){
		throw (new DateFormatException());
	    }
	}
	if ( h < 0 || h> 24){
	    throw (new DateFormatException());
	}
	
	if ( mString.equals("")){
	    throw (new DateFormatException());
	}else {
	    try {
		m = Integer.parseInt(mString.toString());
	    }catch (NumberFormatException ex){
		throw (new DateFormatException());
	    }
	    
	}	
	if ( m < 0 || m> 60){
	    throw (new DateFormatException());
	}
	return h*60+m;
    }
    
    /**
     *  DONE ̃R[obN
     */
    class DonePressed implements ActionListener {
	public DonePressed(){}
	public void actionPerformed(ActionEvent ev){
	    String note = areaNote.getText();
	    String desc = fieldDescription.getText();
	    int start;
	    int end;
	    
	    try {
		start = parseTime(fieldStart.getText());
	    }catch(DateFormatException ex){
		ErrorDialog diag 
		    = new ErrorDialog(fieldStart,
				      ErrMsg.getMsg(ErrMsg.WRN_DATE_FORMAT));
		diag.show();
		return;
	    }
	    
	    try {
		end = parseTime(fieldEnd.getText());
	    }catch(DateFormatException ex){
		ErrorDialog diag
		    = new ErrorDialog(fieldEnd,
				      ErrMsg.getMsg(ErrMsg.WRN_DATE_FORMAT));
		diag.show();
		return;
	    }
	    
	    if ( desc.equals("") ){
		ErrorDialog diag=
		    new ErrorDialog(fieldDescription,
				    ErrMsg.getMsg(ErrMsg.WRN_NULL_FIELD));
		diag.show();
		return;
	    }
	    
	    
	    if ( app == null ){//VK̏ꍇ
		app = new Appointment();
		app.note = note;
		app.description = desc;
		app.start = start;
		app.end = end;
		appList.addElement(app);
	    }else{//ύX̏ꍇ
		app.note = note;
		app.description = desc;
		app.start = start;
		app.end = end;
	    }
	    setVisible(false);
	}
    }
    
    /**
     *  Cancel ̃R[obN
     */
    class CancelPressed implements ActionListener {
	public CancelPressed(){}
	public void actionPerformed(ActionEvent ev){
	    setVisible(false);
	}
    }
    
    /**
     *  Delete ̃R[obN
     */
    class DeletePressed implements ActionListener {
	public DeletePressed(){
	}
	public void actionPerformed(ActionEvent ev){
	    if ( app != null ){
		appList.removeElement(app);
	    }
	    setVisible(false);
	}
    }
    
    /**
     *  RXgN^
     *  @param frame  et[
     */
    AppointEditor(Frame parent){
	super(parent,true);
	
	gbl = new GridBagLayout();
	gbc = new GridBagConstraints();
	panel = new Panel();
	
	place(new Label("Description:"),
	      0,0,
	      1,
	      GridBagConstraints.EAST,GridBagConstraints.NONE);
	
	fieldDescription = new TextField(40);
	
	place(fieldDescription,
	      1,0,
	      3,
	      GridBagConstraints.EAST,GridBagConstraints.BOTH);
	
	place(new Label("Start:"),
	      0,1,
	      1,
	      GridBagConstraints.EAST,GridBagConstraints.NONE);
	
	fieldStart = new TextField(40);
	
	place(fieldStart,
	      1,1,
	      3,
	      GridBagConstraints.EAST,GridBagConstraints.BOTH);
	
	place(new Label("End:"),
	      0,2,
	      1,
	      GridBagConstraints.EAST,GridBagConstraints.NONE);
	
	fieldEnd = new TextField(40);
	
	place(fieldEnd,
	      1,2,
	      3,
	      GridBagConstraints.EAST,GridBagConstraints.BOTH);
	
	place(new Label("Note"),
	      0,3,
	      1,
	      GridBagConstraints.NORTHEAST,GridBagConstraints.NONE);
	
	areaNote = new TextArea(5,60);
	
	place(areaNote,
	      1,3,
	      3,
	      GridBagConstraints.EAST,GridBagConstraints.BOTH);
	
	buttonDone = new Button("Done");
	buttonCancel = new Button("Cancel");
	buttonDelete = new Button("Delete");
	
	buttonDone.addActionListener(new DonePressed());
	buttonCancel.addActionListener(new CancelPressed());
	buttonDelete.addActionListener(new DeletePressed());
	
	Panel control = new Panel();
	control.add(buttonDone);
	control.add(buttonCancel);
	control.add(buttonDelete);
	place(control,
	      1,4,
	      3,
	      GridBagConstraints.EAST,GridBagConstraints.BOTH);
	
	panel.setLayout(gbl);
	add(panel);
	doLayout();
	setSize(600,400);
    }
}


